/*
 * Copyright (C) 2017 Atmark Techno, Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/i2c.h>
#include <linux/delay.h>
#include <linux/gpio.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/of.h>
#include <linux/of_fdt.h>
#include <linux/of_gpio.h>

#include "armadillo_iotg_std_addon.h"

extern_dtb(addon_easel_es920lr_iotg_g3_intf1);
extern_dtb(addon_easel_es920lr_iotg_g3_intf2);
extern_dtb(addon_easel_es920lr_x1_intf1);

#define PIN_NRST	(42)
#define PIN_INT		(46)

int addon_setup_easel_es920lr(struct addon_device *adev)
{
	struct armadillo_iotg_addon *addon = to_addon(adev);
	void *begin;
	size_t size;
	int gpio_nrst;
	int gpio_int;
	char label[64];

	switch (adev->intf) {
	case ADDON_INTERFACE1:
		switch (addon->type) {
		case ADDON_BOARD_TYPE_IOTG:
			begin = dtb_begin(addon_easel_es920lr_iotg_g3_intf1);
			size = dtb_size(addon_easel_es920lr_iotg_g3_intf1);
			break;
		case ADDON_BOARD_TYPE_X1:
			begin = dtb_begin(addon_easel_es920lr_x1_intf1);
			size = dtb_size(addon_easel_es920lr_x1_intf1);
			break;
		default:
			return -ENODEV;
		};
		break;
	case ADDON_INTERFACE2:
		switch (addon->type) {
		case ADDON_BOARD_TYPE_IOTG:
			begin = dtb_begin(addon_easel_es920lr_iotg_g3_intf2);
			size = dtb_size(addon_easel_es920lr_iotg_g3_intf2);
			break;
		default:
			return -ENODEV;
		};
		break;
	default:
		BUG();
	};
	armadillo_iotg_addon_dt_overlay(addon->dev, begin, size);

	gpio_nrst = adev->gpios[PIN_NRST - 1];
	gpio_int = adev->gpios[PIN_INT - 1];

	if (!gpio_is_valid(gpio_nrst)) {
		dev_warn(addon->dev, "nrst is invalid\n");
		return -EINVAL;
	}
	if (devm_gpio_request(addon->dev, gpio_nrst, NULL)) {
		dev_warn(addon->dev, "forceoff request failed\n");
		return -EINVAL;
	}

	if (!gpio_is_valid(gpio_int)) {
		dev_warn(addon->dev, "int is invalid\n");
		return -EINVAL;
	}
	if (devm_gpio_request(addon->dev, gpio_int, NULL)) {
		dev_warn(addon->dev, "ri request failed\n");
		return -EINVAL;
	}

	gpio_direction_output(gpio_nrst, 1);
	gpio_direction_output(gpio_int, 1);

	gpio_export(gpio_nrst, false);
	gpio_export(gpio_int, false);

	snprintf(label, sizeof(label), "NRST_INTF%d", adev->intf);
	gpio_export_link(addon->dev, label, gpio_nrst);
	snprintf(label, sizeof(label), "INT_INTF%d", adev->intf);
	gpio_export_link(addon->dev, label, gpio_int);

	return 0;
}
